@@ -3,14 +3,16 @@
 # ./compile.sh [ i586-pc-msdosdjgpp | i386-pc-cygwin | i386-pc-mingw32 | powerpc-mac-darwin ]
 # Comment the version definition to not compile the library
 VER_E2FSPROGS=
-VER_PROGSREISERFS=0.3.1-rc8
-VER_NTFSPROGS=2.0.0
-VER_LIBEWF=20080501
+VER_PROGSREISERFS="0.3.1-rc8"
+VER_LIBEWF=20100226
+#VER_LIBEWF=20110312
+smp_mflags="-j 2"
 crosscompile_target=
 prefix=/usr/
 if [ "$CC" = "gcc295" ];
 then
   VER_NTFSPROGS=
+  VER_LIBNTFS3G=
 fi
 if [ -z "$1" ];
 then
@@ -19,17 +21,29 @@ else
   compiledir=$1
   if [ "$1" != "$CC" ];
   then
-    VER_E2FSPROGS=1.41.4
+    VER_E2FSPROGS=1.41.8
     crosscompile_target=$1
     TESTDISKCC=$crosscompile_target-gcc
   fi
 fi
+case "$crosscompile_target" in
+  *-msdosdjgpp|*-cygwin|*-mingw32)
+	VER_LIBNTFS3G=
+	VER_NTFSPROGS="2.0.0"
+  ;;
+  *)
+	VER_LIBNTFS3G="2011.3.28-RC"
+	VER_NTFSPROGS=
+  ;;
+esac
 prefix=/usr/$crosscompile_target
 LYNX=links
+WGET="wget -N"
 LIBEXT=$compiledir/e2fsprogs-$VER_E2FSPROGS/lib/ext2fs/libext2fs.a
 LIBNTFS=$compiledir/ntfsprogs-$VER_NTFSPROGS/libntfs/.libs/libntfs.a
+LIBNTFS3G=$compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G/libntfs-3g/.libs/libntfs-3g.a
 LIBREISER=$compiledir/progsreiserfs-$VER_PROGSREISERFS/libreiserfs/.libs/libreiserfs.a
-LIBEWF=$compiledir/ewf-$VER_NTFSPROGS/libewf/.libs/libewf.a
+LIBEWF=$compiledir/ewf-$VER_LIBEWF/libewf/.libs/libewf.a
 pwd_saved=`pwd`
 confdir=`(dirname "$0") 2>/dev/null`
 cd $confdir
@@ -63,7 +77,14 @@ then
   then
         rm -f $compiledir/Makefile
   	cd $compiledir/e2fsprogs-$VER_E2FSPROGS
-	CC=$TESTDISKCC CFLAGS="$CFLAGS -g -O2 -DOMIT_COM_ERR" ./configure --host=$crosscompile_target --prefix=$prefix
+        case "$crosscompile_target" in
+	  arm-marvell-linux-gnu)
+	  	CC=$TESTDISKCC CFLAGS="$CFLAGS -g -O2 -DOMIT_COM_ERR" ./configure --host=$crosscompile_target --prefix=$prefix --disable-tls
+                ;;
+          *)
+		CC=$TESTDISKCC CFLAGS="$CFLAGS -g -O2 -DOMIT_COM_ERR" ./configure --host=$crosscompile_target --prefix=$prefix
+                ;;
+	esac
 	cd $pwd_saved
   fi
 fi
@@ -73,7 +94,7 @@ then
   if [ -e $compiledir/e2fsprogs-$VER_E2FSPROGS/Makefile ];
   then
 	cd $compiledir/e2fsprogs-$VER_E2FSPROGS
-	make libs
+	make $smp_mflags libs
 	cd $pwd_saved
   fi
 fi
@@ -91,6 +112,10 @@ then
   if [ -e progsreiserfs-$VER_PROGSREISERFS.tar.gz ];
   then
         tar xzf progsreiserfs-$VER_PROGSREISERFS.tar.gz -C $compiledir
+        cd $compiledir/progsreiserfs-$VER_PROGSREISERFS
+	patch -p1 < $pwd_saved/progsreiserfs-journal.patch
+	patch -p1 < $pwd_saved/progsreiserfs-file-read.patch
+        cd $pwd_saved
   fi
 fi
 
@@ -112,7 +137,53 @@ then
   if [ -e $compiledir/progsreiserfs-$VER_PROGSREISERFS/Makefile ];
   then
 	cd $compiledir/progsreiserfs-$VER_PROGSREISERFS
-	make
+	make $smp_mflags
+	cd $pwd_saved
+  fi
+fi
+fi
+
+if [ "$VER_LIBNTFS3G" != "" ];
+then
+CONFIGUREOPT="$CONFIGUREOPT --with-ntfs3g-lib=${PWDSRC}/ntfs-3g_ntfsprogs-${VER_LIBNTFS3G}/libntfs-3g/.libs/ --with-ntfs3g-includes=${PWDSRC}/ntfs-3g_ntfsprogs-${VER_LIBNTFS3G}/include/"
+if [ ! -e $compiledir/ntfsprogs-$VER_LIBNTFS3G/configure ];
+then
+  if [ ! -e ntfs-3g_ntfsprogs-$VER_LIBNTFS3G.tgz ];
+  then
+	$WGET http://tuxera.com/opensource/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G.tgz
+  fi
+  if [ -e ntfs-3g_ntfsprogs-$VER_LIBNTFS3G.tgz ];
+  then
+	tar xzf ntfs-3g_ntfsprogs-$VER_LIBNTFS3G.tgz -C $compiledir
+  fi
+fi
+
+if [ ! -e $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G/Makefile ];
+then
+  if [ -e $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G/configure ];
+  then
+#        rm -f $compiledir/Makefile
+        cd $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G
+        case "$crosscompile_target" in
+          powerpc-apple-darwin|i686-apple-darwin9)
+	  	CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-default-device-io-ops --disable-crypto --disable-nfconv
+		;;
+	*)
+		CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-default-device-io-ops --disable-crypto
+		;;
+	esac
+# --disable-default-device-io-ops is need for NT 4
+        cd $pwd_saved
+  fi
+fi
+
+if [ ! -e $VER_LIBNTFS3G ];
+then
+  if [ -e $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G/Makefile ];
+  then
+	cd $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G
+#	make $smp_mflags libs
+	make $smp_mflags
 	cd $pwd_saved
   fi
 fi
@@ -150,7 +221,7 @@ then
   if [ -e $compiledir/ntfsprogs-$VER_NTFSPROGS/Makefile ];
   then
 	cd $compiledir/ntfsprogs-$VER_NTFSPROGS
-	make libs
+	make $smp_mflags libs
 	cd $pwd_saved
   fi
 fi
@@ -162,17 +233,18 @@ CONFIGUREOPT="$CONFIGUREOPT --with-ewf-lib=${PWDSRC}/libewf-${VER_LIBEWF}/libewf
 
 if [ ! -e $compiledir/libewf-$VER_LIBEWF/configure ];
 then
-  if [ ! -e libewf-$VER_LIBEWF.tar.gz ];
-  then
-    if [ ! -e libewf-beta-$VER_LIBEWF.tar.gz ];
+  if [ ! -e libewf-$VER_LIBEWF.tar.gz -a ! -e libewf-alpha-$VER_LIBEWF.tar.gz -a ! -e libewf-beta-$VER_LIBEWF.tar.gz ];
     then
-      	$LYNX "http://sourceforge.net/project/platformdownload.php?group_id=167783"
-    fi
+  	$LYNX "http://sourceforge.net/project/platformdownload.php?group_id=167783"
   fi
   if [ -e libewf-$VER_LIBEWF.tar.gz ];
   then
 	tar xzf libewf-$VER_LIBEWF.tar.gz -C $compiledir
   fi
+  if [ -e libewf-alpha-$VER_LIBEWF.tar.gz ];
+  then
+	tar xzf libewf-alpha-$VER_LIBEWF.tar.gz -C $compiledir
+  fi
   if [ -e libewf-beta-$VER_LIBEWF.tar.gz ];
   then
 	tar xzf libewf-beta-$VER_LIBEWF.tar.gz -C $compiledir
@@ -195,7 +267,7 @@ then
   if [ -e $compiledir/libewf-$VER_LIBEWF/Makefile ];
   then
 	cd $compiledir/libewf-$VER_LIBEWF
-	make lib
+	make $smp_mflags lib
 	cd $pwd_saved
   fi
 fi
@@ -217,15 +289,24 @@ then
 # but users may be unable to navigate...
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --enable-sudo --with-sudo-bin=/usr/bin/sudo
                 ;;
+	  i686-apple-darwin9)
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-sudo --with-sudo-bin=/usr/bin/sudo
+                ;;
           i586-pc-msdosdjgpp)
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-iconv
                 ;;
           i386-pc-cygwin)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-iconv
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT
                 ;;
           i386-mingw32)
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-iconv --enable-missing-uuid-ok
                 ;;
+	  i686-pc-mingw32|x86_64-pc-mingw32)
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-missing-uuid-ok
+                ;;
+	  arm-marvell-linux-gnu)
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-ntfs
+                ;;
           *)
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT
                 ;;
@@ -235,8 +316,7 @@ then
   if [ -e $compiledir/Makefile ];
   then
   	cd $compiledir
-        make
+        make $smp_mflags
 	cd $pwd_saved
   fi
 fi
-
