@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -36,15 +37,11 @@
 #include <sys/stat.h>
 #endif
 #include "types.h"
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
 #include <errno.h>
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
 #include "common.h"
-#include <stdio.h>
 #include "dir.h"
 #include "log.h"
 #include "log_part.h"
@@ -164,21 +161,16 @@ int dir_aff_log(const dir_data_t *dir_data, const file_data_t*dir_list)
   {
     char		datestr[80];
     char str[11];
-    if((current_file->status&FILE_STATUS_DELETED)!=0)
-      log_info("X");
-    else
-      log_info(" ");
     if(current_file->stat.st_mtime)
     {
       struct tm		*tm_p;
       tm_p = localtime(&current_file->stat.st_mtime);
-
       snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
 	  tm_p->tm_mday, monstr[tm_p->tm_mon],
 	  1900 + tm_p->tm_year, tm_p->tm_hour,
 	  tm_p->tm_min);
       /* FIXME: a check using current_file->name will be better */
-      if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2012)
+      if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
       {
 	test_date=1;
       }
@@ -186,12 +178,20 @@ int dir_aff_log(const dir_data_t *dir_data, const file_data_t*dir_list)
       strncpy(datestr, "                 ",sizeof(datestr));
     }
     mode_string(current_file->stat.st_mode,str);
-    log_info("%7lu %s %5u  %5u   %7llu %s ",
+    if((current_file->status&FILE_STATUS_DELETED)!=0)
+      log_info("X");
+    else
+      log_info(" ");
+    log_info("%7lu %s %5u  %5u %9llu %s ",
 	(unsigned long int)current_file->stat.st_ino,
 	str,
 	(unsigned int)current_file->stat.st_uid,
 	(unsigned int)current_file->stat.st_gid,
+#ifdef DJGPP
+	(long long unsigned int)current_file->file_size,
+#else
 	(long long unsigned int)current_file->stat.st_size,
+#endif
 	datestr);
     if(dir_data!=NULL && (dir_data->param&FLAG_LIST_PATHNAME)!=0)
     {
@@ -243,9 +243,13 @@ int log_list_file(const disk_t *disk, const partition_t *partition, const dir_da
     }
     mode_string(current_file->stat.st_mode,str);
     log_info("%7lu ",(unsigned long int)current_file->stat.st_ino);
-    log_info("%s %5u  %5u   ", 
+    log_info("%s %5u %5u ", 
 	str, (unsigned int)current_file->stat.st_uid, (unsigned int)current_file->stat.st_gid);
-    log_info("%7llu", (long long unsigned int)current_file->stat.st_size);
+#ifdef DJGPP
+    log_info("%9llu", (long long unsigned int)current_file->file_size);
+#else
+    log_info("%9llu", (long long unsigned int)current_file->stat.st_size);
+#endif
     log_info(" %s %s\n", datestr, current_file->name);
   }
   return test_date;
@@ -273,6 +277,7 @@ void delete_list_file_info(struct td_list_head *list)
   {
     file_info_t *file_info;
     file_info=td_list_entry(file_walker, file_info_t, list);
+    free(file_info->name);
     td_list_del(file_walker);
     free(file_info);
   }
@@ -331,35 +336,6 @@ int dir_whole_partition_log(disk_t *disk, const partition_t *partition, dir_data
   return dir_whole_partition_log_aux(disk, partition, dir_data, inode);
 }
 
-/**
- * set_date - Set the file's date and time
- * @pathname:  Path and name of the file to alter
- * @actime:    Date and time to set
- * @modtime:   Date and time to set
- *
- * Give a file a particular date and time.
- *
- * Return:  1  Success, set the file's date and time
- *	    0  Error, failed to change the file's date and time
- */
-int set_date(const char *pathname, time_t actime, time_t modtime)
-{
-#ifdef HAVE_UTIME
-  struct utimbuf ut;
-  if (!pathname)
-    return 0;
-  ut.actime  = actime;
-  ut.modtime = modtime;
-  if (utime(pathname, &ut)) {
-    log_error("ERROR: Couldn't set the file's date and time for %s\n", pathname);
-    return 0;
-  }
-  return 1;
-#else
-  return 0;
-#endif
-}
-
 int filesort(const struct td_list_head *a, const struct td_list_head *b)
 {
   const file_info_t *file_a=td_list_entry(a, file_info_t, list);
@@ -369,13 +345,13 @@ int filesort(const struct td_list_head *a, const struct td_list_head *b)
   if(res)
     return res;
   /* . and .. must listed before the other directories */
-  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name,".")==0)
+  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, ".")==0)
     return -1;
-  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name,"..")==0 &&
-      !strcmp(file_b->name,".")==0)
+  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, "..")==0 &&
+      !strcmp(file_b->name, ".")==0)
     return -1;
   /* Files and directories are sorted by name */
-  return strcmp(file_a->name,file_b->name);
+  return strcmp(file_a->name, file_b->name);
 }
 
 /*
@@ -646,7 +622,6 @@ char *mkdir_local(const char *localroot, const char *pathname)
   while(*src!='\0')
   {
     unsigned int n=0;
-    unsigned int l;
     const char *src_org=src;
     char *dst_org=dst;
     for(n=0;
@@ -657,6 +632,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
 #ifdef __MINGW32__
     if(mkdir(localdir)<0 && errno==EINVAL)
     {
+      unsigned int l;
       l=filename_convert(dst_org, src_org, n);
       dst=dst_org+l;
       mkdir(localdir);
@@ -665,6 +641,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
     if(memcmp(&localdir[1],":/cygdrive",11)!=0 &&
 	mkdir(localdir, 0775)<0 && errno==EINVAL)
     {
+      unsigned int l;
       l=filename_convert(dst_org, src_org, n);
       dst=dst_org+l;
       mkdir(localdir, 0775);
@@ -672,6 +649,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
 #else
     if(mkdir(localdir, 0775)<0 && errno==EINVAL)
     {
+      unsigned int l;
       l=filename_convert(dst_org, src_org, n);
       dst=dst_org+l;
       mkdir(localdir, 0775);
@@ -739,7 +717,8 @@ FILE *fopen_local(char **localfilename, const char *localroot, const char *filen
       }
 #elif defined(__CYGWIN__)
       if(memcmp(&localfilename[1],":/cygdrive",11)!=0 &&
-	  mkdir(*localfilename, 0775)<0 && errno==EINVAL)
+	  mkdir(*localfilename, 0775)<0 &&
+	  (errno==EINVAL || errno==ENOENT))
       {
 	unsigned int l;
 	l=filename_convert(dst_org, src_org, n);
