@@ -50,6 +50,7 @@
 #include "fat_cluster.h"
 #include "fat_unformat.h"
 #include "pnext.h"
+#include "setdate.h"
 
 #define READ_SIZE 1024*512
 static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const int verbose, const int interface, unsigned int *sectors_per_cluster, uint64_t *offset_org, alloc_data_t *list_search_space)
@@ -90,7 +91,7 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
     if(memcmp(buffer,         ".          ", 8+3)==0 &&
 	memcmp(&buffer[0x20], "..         ", 8+3)==0)
     {
-      unsigned long int cluster=(buffer[0*0x20+0x15]<<24) + (buffer[0*0x20+0x14]<<16) +
+      const unsigned long int cluster=(buffer[0*0x20+0x15]<<24) + (buffer[0*0x20+0x14]<<16) +
 	(buffer[0*0x20+0x1B]<<8) + buffer[0*0x20+0x1A];
       log_info("sector %lu, cluster %lu\n",
 	  (unsigned long)(offset/disk->sector_size), cluster);
@@ -107,7 +108,9 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
       buffer=buffer_start;
       if(verbose>1)
       {
-        log_verbose("Reading sector %10lu/%lu\n",(unsigned long)((offset-partition->part_offset)/disk->sector_size),(unsigned long)((partition->part_size-1)/disk->sector_size));
+        log_verbose("Reading sector %10llu/%llu\n",
+	    (unsigned long long)((offset-partition->part_offset)/disk->sector_size),
+	    (unsigned long long)((partition->part_size-1)/disk->sector_size));
       }
       if(disk->pread(disk, buffer_start, READ_SIZE, offset) != READ_SIZE)
       {
@@ -123,12 +126,16 @@ static int fat_copy_file(disk_t *disk, const partition_t *partition, const unsig
   char *new_file;	
   FILE *f_out;
   unsigned int cluster;
+#ifdef DJGPP
+  unsigned int file_size=file->file_size;
+#else
   unsigned int file_size=file->stat.st_size;
+#endif
   unsigned long int no_of_cluster;
   unsigned char *buffer_file=(unsigned char *)MALLOC(block_size);
   cluster = file->stat.st_ino;
   new_file=(char *)MALLOC(1024);
-  snprintf(new_file, 1024, "%s.%u/f%u-%s", recup_dir, dir_num,
+  snprintf(new_file, 1024, "%s.%u/f%07u-%s", recup_dir, dir_num,
       (unsigned int)((start_data - partition->part_offset + (uint64_t)(cluster-2)*block_size)/disk->sector_size),
       file->name);
   log_info("fat_copy_file %s\n", new_file);
@@ -172,34 +179,35 @@ static int fat_copy_file(disk_t *disk, const partition_t *partition, const unsig
 
 static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verbose, const char *recup_dir, const int interface, unsigned int *file_nbr, const unsigned int blocksize, const uint64_t start_offset, alloc_data_t *list_search_space, const time_t real_start_time, unsigned int *dir_num)
 {
-  uint64_t offset=0;
+  uint64_t offset;
+  uint64_t offset_end;
   unsigned char *buffer_start;
-  unsigned char *buffer_olddata;
   unsigned char *buffer;
   time_t start_time;
   time_t previous_time;
-  unsigned int buffer_size;
   const unsigned int read_size=(blocksize>65536?blocksize:65536);
   alloc_data_t *current_search_space;
   file_recovery_t file_recovery;
-  buffer_size=blocksize+READ_SIZE;
-  buffer_start=(unsigned char *)MALLOC(buffer_size);
-  buffer_olddata=buffer_start;
-  buffer=buffer_olddata+blocksize;
   reset_file_recovery(&file_recovery);
+  file_recovery.blocksize=blocksize;
+  buffer_start=(unsigned char *)MALLOC(READ_SIZE);
+  buffer=buffer_start;
   start_time=time(NULL);
   previous_time=start_time;
-  memset(buffer_olddata,0,blocksize);
+  current_search_space=td_list_entry(list_search_space->list.prev, alloc_data_t, list);
+  if(current_search_space==list_search_space)
+  {
+    free(buffer_start);
+    return 0;
+  }
+  offset_end=current_search_space->end;
   current_search_space=td_list_entry(list_search_space->list.next, alloc_data_t, list);
-  if(current_search_space!=list_search_space)
-    offset=current_search_space->start;
+  offset=current_search_space->start;
   if(verbose>0)
     info_list_search_space(list_search_space, current_search_space, disk->sector_size, 0, verbose);
   disk->pread(disk, buffer, READ_SIZE, offset);
-  while(current_search_space!=list_search_space)
+  for(;offset < offset_end; offset+=blocksize)
   {
-    const uint64_t old_offset=offset;
-    get_next_sector(list_search_space, &current_search_space,&offset,blocksize);
     if(memcmp(buffer,         ".          ", 8+3)==0 &&
 	memcmp(&buffer[0x20], "..         ", 8+3)==0)
     {
@@ -208,9 +216,9 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
       if(dir_list!=NULL)
       {
 	const file_data_t *current_file;
-	log_info("Sector %llu\n", (long long unsigned)old_offset/disk->sector_size);
+	log_info("Sector %llu\n", (long long unsigned)offset/disk->sector_size);
 	dir_aff_log(NULL, dir_list);
-	del_search_space(list_search_space, old_offset, old_offset + blocksize -1);
+	del_search_space(list_search_space, offset, offset + blocksize -1);
 	current_file=dir_list;
 	while(current_file!=NULL)
 	{
@@ -222,11 +230,16 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
 	      LINUX_S_ISREG(current_file->stat.st_mode)!=0)
 	  {
 	    const uint64_t file_start=start_offset + (uint64_t)(current_file->stat.st_ino - 2) * blocksize;
+#ifdef DJGPP
+	    const uint64_t file_end=file_start+(current_file->file_size+blocksize-1)/blocksize*blocksize - 1;
+#else
 	    const uint64_t file_end=file_start+(current_file->stat.st_size+blocksize-1)/blocksize*blocksize - 1;
+#endif
 	    if(file_end < partition->part_offset + partition->part_size)
 	    {
 	      if(fat_copy_file(disk, partition, blocksize, start_offset, recup_dir, *dir_num, current_file)==0)
 	      {
+		(*file_nbr)++;
 		del_search_space(list_search_space, file_start, file_end);
 	      }
 	      current_file=current_file->next;
@@ -238,25 +251,17 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
 	    current_file=current_file->next;
 	}
 	delete_list_file(dir_list);
-	(*file_nbr)++;
       }
     }
-    if(current_search_space==list_search_space)
-    {
-      /* End of disk found => EOF */
-      reset_file_recovery(&file_recovery);
-    }
-    buffer_olddata+=blocksize;
     buffer+=blocksize;
-    if( old_offset+blocksize!=offset ||
-        buffer+read_size>buffer_start+buffer_size)
+    if(buffer+read_size>buffer_start+READ_SIZE)
     {
-      memcpy(buffer_start,buffer_olddata,blocksize);
-      buffer_olddata=buffer_start;
-      buffer=buffer_olddata+blocksize;
+      buffer=buffer_start;
       if(verbose>1)
       {
-        log_verbose("Reading sector %10lu/%lu\n",(unsigned long)((offset-partition->part_offset)/disk->sector_size),(unsigned long)((partition->part_size-1)/disk->sector_size));
+        log_verbose("Reading sector %10llu/%llu\n",
+	    (unsigned long long)((offset-partition->part_offset)/disk->sector_size),
+	    (unsigned long long)((partition->part_size-1)/disk->sector_size));
       }
       if(disk->pread(disk, buffer, READ_SIZE, offset) != READ_SIZE)
       {
@@ -281,9 +286,12 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
           previous_time=current_time;
 	  wmove(stdscr,9,0);
 	  wclrtoeol(stdscr);
-	  wprintw(stdscr,"Reading sector %10lu/%lu, %u files found\n",
-	      (unsigned long)((offset-partition->part_offset)/disk->sector_size),
-	      (unsigned long)(partition->part_size/disk->sector_size), file_nbr);
+	  log_info("Reading sector %10llu/%llu, %u files found\n",
+	      (unsigned long long)((offset-partition->part_offset)/disk->sector_size),
+	      (unsigned long long)(partition->part_size/disk->sector_size), *file_nbr);
+	  wprintw(stdscr,"Reading sector %10llu/%llu, %u files found\n",
+	      (unsigned long long)((offset-partition->part_offset)/disk->sector_size),
+	      (unsigned long long)(partition->part_size/disk->sector_size), *file_nbr);
 	  wmove(stdscr,10,0);
 	  wclrtoeol(stdscr);
 	  wprintw(stdscr,"Elapsed time %uh%02um%02us",
@@ -292,7 +300,7 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
 	      (unsigned)(elapsed_time%60));
 	  if(offset-partition->part_offset!=0)
 	  {
-	    wprintw(stdscr," - Estimated time for achievement %uh%02um%02u\n",
+	    wprintw(stdscr," - Estimated time to completion %uh%02um%02u\n",
 		(unsigned)((partition->part_offset+partition->part_size-1-offset)*elapsed_time/(offset-partition->part_offset)/3600),
 		(unsigned)(((partition->part_offset+partition->part_size-1-offset)*elapsed_time/(offset-partition->part_offset)/60)%60),
 		(unsigned)((partition->part_offset+partition->part_size-1-offset)*elapsed_time/(offset-partition->part_offset))%60);
@@ -301,13 +309,13 @@ static int fat_unformat_aux(disk_t *disk, partition_t *partition, const int verb
 	  if(check_enter_key_or_s(stdscr))
 	  {
 	    log_info("PhotoRec has been stopped\n");
-	    current_search_space=list_search_space;
+	    offset = offset_end;
 	  }
 	}
       }
 #endif
     }
-  } /* end while(current_search_space!=list_search_space) */
+  }
   free(buffer_start);
   return 0;
 }
