@@ -2,7 +2,7 @@
 
     File: file_zip.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2009 Christophe GRENIER <grenier@cgsecurity.org>
     Copyright (C) 2007      Christophe GISQUET <christophe.gisquet@free.fr>
 
     This software is free software; you can redistribute it and/or modify
@@ -45,6 +45,8 @@ static void register_header_check_zip(file_stat_t *file_stat);
 static int header_check_zip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_zip(file_recovery_t *file_recovery);
 static unsigned int pos_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size);
+static void file_rename_zip(const char *old_filename);
+static char first_filename[256];
 
 const file_hint_t file_hint_zip= {
   .extension="zip",
@@ -131,7 +133,7 @@ static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size
   return -1;
 }
 
-static int zip_parse_file_entry(file_recovery_t *fr)
+static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const unsigned int file_nbr)
 {
   zip_file_entry_t  file;
   uint32_t          len;
@@ -165,9 +167,57 @@ static int zip_parse_file_entry(file_recovery_t *fr)
       return -1;
     }
     filename[len]='\0';
+    if(first_filename[0]=='\0')
+    {
+      const unsigned int len_tmp=(len<255?len:255);
+      strncpy(first_filename, filename, len_tmp);
+      first_filename[len_tmp]='\0';
+    }
 #ifdef DEBUG_ZIP
-    log_info("%s", filename);
+    log_info("%s\n", filename);
 #endif
+    if(*ext==NULL)
+    {
+      static int msoffice=0;
+      if(file_nbr==0)
+      {
+	msoffice=0;
+	if(len==36 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.calc", 36)==0)
+	  *ext="sxc";
+	else if(len==36 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.draw", 36)==0)
+	  *ext="sxd";
+	else if(len==39 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.impress", 39)==0)
+	  *ext="sxi";
+	else if(len==38 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.writer", 38)==0)
+	  *ext="sxw";
+	else if(len==51 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.graphics",51)==0)
+	  *ext="odg";
+	else if(len==55 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.presentation",55)==0)
+	  *ext="odp";
+	else if(len==54 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.spreadsheet",54)==0)
+	  *ext="ods";
+	else if(len==47 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.text",47)==0)
+	  *ext="odt";
+	else if(len==19 && memcmp(filename, "[Content_Types].xml", 19)==0)
+	  msoffice=1;
+      }
+      else if(file_nbr==2 && msoffice!=0)
+      {
+	if(strncmp(filename, "word/", 5)==0)
+	  *ext="docx";
+	else if(strncmp(filename, "xl/", 3)==0)
+	  *ext="xlsx";
+	else if(strncmp(filename, "ppt/", 4)==0)
+	  *ext="pptx";
+      }
+    }
+    if(*ext==NULL)
+    {
+      if(len==20 && strcasecmp(filename, "META-INF/MANIFEST.MF")==0)
+	*ext="jar";
+      else if(len==15 && strcasecmp(filename, "chrome.manifest")==0)
+	*ext="xpi";
+    }
     free(filename);
   }
 #ifdef DEBUG_ZIP
@@ -441,12 +491,16 @@ static int zip64_parse_end_central_dir_locator(file_recovery_t *fr)
 
 static void file_check_zip(file_recovery_t *fr)
 {
+  const char *ext=NULL;
+  unsigned int file_nbr=0;
   fseek(fr->handle, 0, SEEK_SET);
   fr->file_size = 0;
   fr->offset_error=0;
-
+  fr->offset_ok=0;
+  first_filename[0]='\0';
   while (1)
   {
+    uint64_t file_size_old;
     uint32_t header;
     int      status;
 
@@ -466,6 +520,7 @@ static void file_check_zip(file_recovery_t *fr)
     log_flush();
 #endif
     fr->file_size += 4;
+    file_size_old=fr->file_size;
 
     switch (header)
     {
@@ -485,7 +540,8 @@ static void file_check_zip(file_recovery_t *fr)
         status = zip_parse_data_desc(fr);
         break;
       case ZIP_FILE_ENTRY: /* File Entry */
-        status = zip_parse_file_entry(fr);
+        status = zip_parse_file_entry(fr, &ext, file_nbr);
+	file_nbr++;
         break;
       case ZIP_SIGNATURE: /* Signature */
         status = zip_parse_signature(fr);
@@ -511,6 +567,105 @@ static void file_check_zip(file_recovery_t *fr)
     /* Only end of central dir is end of archive, 64b version of it is before */
     if (header==ZIP_END_CENTRAL_DIR)
       return;
+    fr->offset_ok=file_size_old;
+  }
+}
+
+static void file_rename_zip(const char *old_filename)
+{
+  const char *ext=NULL;
+  unsigned int file_nbr=0;
+  file_recovery_t fr;
+  reset_file_recovery(&fr);
+  if((fr.handle=fopen(old_filename, "rb"))==NULL)
+    return;
+  fseek(fr.handle, 0, SEEK_SET);
+  fr.file_size = 0;
+  fr.offset_error=0;
+  first_filename[0]='\0';
+
+  while (1)
+  {
+    uint32_t header;
+    int      status;
+
+    if (fread(&header, 4, 1, fr.handle)!=1)
+    {
+#ifdef DEBUG_ZIP
+      log_trace("Failed to read block header\n");
+#endif
+      fclose(fr.handle);
+      return;
+    }
+
+    header = le32(header);
+#ifdef DEBUG_ZIP
+    log_trace("Header 0x%08X at 0x%llx\n", header, (long long unsigned int)fr.file_size);
+    log_flush();
+#endif
+    fr.file_size += 4;
+
+    switch (header)
+    {
+      case ZIP_CENTRAL_DIR: /* Central dir */
+        status = zip_parse_central_dir(&fr);
+        break;
+      case ZIP_CENTRAL_DIR64: /* 64b end central dir */
+        status = zip64_parse_end_central_dir(&fr);
+        break;
+      case ZIP_END_CENTRAL_DIR: /* End central dir */
+        status = zip_parse_end_central_dir(&fr);
+        break;
+      case ZIP_END_CENTRAL_DIR64: /* 64b end central dir locator */
+        status = zip64_parse_end_central_dir_locator(&fr);
+        break;
+      case ZIP_DATA_DESCRIPTOR: /* Data descriptor */
+        status = zip_parse_data_desc(&fr);
+        break;
+      case ZIP_FILE_ENTRY: /* File Entry */
+        status = zip_parse_file_entry(&fr, &ext, file_nbr);
+	file_nbr++;
+	if(ext!=NULL)
+	{
+	  fclose(fr.handle);
+	  file_rename(old_filename, NULL, 0, 0, ext, 1);
+	  return;
+	}
+        break;
+      case ZIP_SIGNATURE: /* Signature */
+        status = zip_parse_signature(&fr);
+        break;
+      default:
+#ifdef DEBUG_ZIP
+        if ((header&0xFFFF) != 0x4B50)
+          log_trace("Not a zip block: 0x%08X\n", header);
+        else
+          log_trace("Unparsable block with ID 0x%04X\n", header>>16);
+#endif
+        status = -1;
+        break;
+    }
+
+    /* Verify status */
+    if (status<0)
+    {
+      fclose(fr.handle);
+      return;
+    }
+    /* Only end of central dir is end of archive, 64b version of it is before */
+    if (header==ZIP_END_CENTRAL_DIR)
+    {
+      unsigned int len;
+      fclose(fr.handle);
+      for(len=0; len<32 &&
+	  first_filename[len]!='\0' &&
+	  first_filename[len]!='.' &&
+	  first_filename[len]!='/' &&
+	  first_filename[len]!='\\';
+	  len++);
+      file_rename(old_filename, first_filename, len, 0, "zip", 0);
+      return;
+    }
   }
 }
 
@@ -563,17 +718,24 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
        file_recovery_new->extension="odt";
      else if(memcmp(&buffer[30],"[Content_Types].xml",19)==0)
      {
-       if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"word", 4)!=0)
+       if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"word/", 5)!=0)
          file_recovery_new->extension="docx";
-       else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl", 2)!=0)
+       else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl/", 3)!=0)
          file_recovery_new->extension="xlsx";
-       else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"ppt", 3)!=0)
+       else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"ppt/", 4)!=0)
          file_recovery_new->extension="pptx";
        else
          file_recovery_new->extension="docx";
+       file_recovery_new->file_rename=&file_rename_zip;
      }
+     /* Extended Renoise song file */
+    else if(memcmp(&buffer[30], "Song.xml", 8)==0)
+      file_recovery_new->extension="xrns";
     else
+    {
       file_recovery_new->extension=file_hint_zip.extension;
+      file_recovery_new->file_rename=&file_rename_zip;
+    }
     return 1;
   }
   else if(memcmp(buffer,zip_header2,sizeof(zip_header2))==0)
