@@ -108,12 +108,40 @@
 #include "alignio.h"
 #include "hpa_dco.h"
 
-extern const arch_fnct_t arch_gpt;
-extern const arch_fnct_t arch_mac;
+#if defined(HAVE_PREAD) && defined(TARGET_LINUX)
+//#define HDCLONE 1
+#endif
+
+struct tdewf_file_header
+{
+        /* The EWF file signature (magic header)
+         * consists of 8 bytes containing
+         * EVF 0x09 0x0d 0x0a 0xff 0x00
+         */
+        uint8_t signature[ 8 ];
+        /* The fields start
+         * consists of 1 byte (8 bit) containing
+         * 0x01
+         */
+        uint8_t fields_start;
+        /* The fields segment number
+         * consists of 2 bytes (16 bits) containing
+         */
+        uint16_t fields_segment;
+        /* The fields end
+         * consists of 2 bytes (16 bits) containing
+         * 0x00 0x00
+         */
+        uint16_t fields_end;
+} __attribute__ ((__packed__));
+
 
 struct info_file_struct
 {
   int handle;
+#ifdef HDCLONE
+  int handle_clone;
+#endif
   char file_name[DISKNAME_MAX];
   int mode;
 };
@@ -123,6 +151,7 @@ static const char *file_description(disk_t *disk_car);
 static const char *file_description_short(disk_t *disk_car);
 static int file_clean(disk_t *disk_car);
 static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset);
+static void *file_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
 static int file_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_sync(disk_t *disk_car);
@@ -250,18 +279,17 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
   }
 #elif defined(__APPLE__)
   {
-    char device_scsi[]="/dev/disk0";
-    char device_raw[]="/dev/rdisk0";
+    char device[100];
     /* Disk */
-    for(i=0;i<10;i++)
+    for(i=0;i<20;i++)
     {
-      device_scsi[strlen(device_scsi)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,verbose,arch,testdisk_mode));
+      snprintf(device, sizeof(device), "/dev/disk%u", i);
+      list_disk=insert_new_disk(list_disk,file_test_availability(device, verbose, arch, testdisk_mode));
     }
-    for(i=0;i<10;i++)
+    for(i=0;i<20;i++)
     {
-      device_raw[strlen(device_raw)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_raw,verbose,arch,testdisk_mode));
+      snprintf(device, sizeof(device), "/dev/rdisk%u", i);
+      list_disk=insert_new_disk(list_disk,file_test_availability(device, verbose, arch, testdisk_mode));
     }
   }
 #elif defined(TARGET_LINUX)
@@ -353,6 +381,12 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
       }
     }
   }
+#elif defined(__HAIKU__)
+  {
+#ifdef HAVE_GLOB_H
+    list_disk=hd_glob_parse("/dev/disk/*/*/master/raw", list_disk, verbose, arch, testdisk_mode);
+#endif
+  }
 #else
   {
     /* Need to check http://mattriffle.com/mirrors/freebsd/doc/en_US.ISO8859-1/books/handbook/disks-naming.html#DISK-NAMING-PHYSICAL-TABLE */
@@ -366,6 +400,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     char device_ide_hd[]="/dev/ad0";
     char device_scsi_hd[]="/dev/da0";
     char device_cd[]="/dev/acd0";
+    char device_vnd[]="/dev/rsvnd0c";	/* virtual node driver, interface to a disk image file */
     /* wd da */
     /* Disk IDE */
     for(i=0;i<8;i++)
@@ -401,7 +436,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     }
     for(i=0;i<8;i++)
     {
-      device_scsi2[strlen(device_scsi2)-1]='0'+i;
+      device_scsi2[strlen(device_scsi2)-2]='0'+i;
       list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi2,verbose,arch,testdisk_mode));
     }
     for(i=0;i<8;i++)
@@ -421,6 +456,12 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
       device_cd[strlen(device_cd)-1]='0'+i;
       list_disk=insert_new_disk(list_disk,file_test_availability(device_cd,verbose,arch,testdisk_mode));
     }
+    /* VND */
+    for(i=0;i<4;i++)
+    {
+      device_vnd[strlen(device_vnd)-2]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_vnd,verbose,arch,testdisk_mode));
+    }
   }
 #endif
   return list_disk;
@@ -509,7 +550,8 @@ static unsigned int disk_get_sector_size(const int hd_h, const char *device, con
       {
 	log_verbose("disk_get_sector_size GetDiskFreeSpaceA %s Ok\n",device);
       }
-      if(sector_size!=0)
+      /* sector_size <= 16MB*/
+      if(sector_size>0 && sector_size <= (1<<24))
 	return sector_size;
     }
   }
@@ -792,21 +834,24 @@ static char* strip_name(char* str)
 /* This function reads the /sys entry named "file" for device "disk_car". */
 static char * read_device_sysfs_file (const disk_t *disk_car, const char *file)
 {
-        FILE *f;
-        char name_buf[128];
-        char buf[256];
+  FILE *f;
+  char name_buf[128];
+  char buf[256];
 
-        snprintf (name_buf, 127, "/sys/block/%s/device/%s",
-                  basename (disk_car->device), file);
+  snprintf (name_buf, 127, "/sys/block/%s/device/%s",
+      basename (disk_car->device), file);
 
-        if ((f = fopen (name_buf, "r")) == NULL)
-                return NULL;
+  if ((f = fopen (name_buf, "r")) == NULL)
+    return NULL;
 
-        if (fgets (buf, 255, f) == NULL)
-                return NULL;
+  if (fgets (buf, 255, f) == NULL)
+  {
+    fclose (f);
+    return NULL;
+  }
 
-        fclose (f);
-        return strip_name (buf);
+  fclose (f);
+  return strip_name (buf);
 }
 #endif
 
@@ -1023,8 +1068,9 @@ static const char *file_description(disk_t *disk_car)
 {
   const struct info_file_struct *data=(const struct info_file_struct *)disk_car->data;
   char buffer_disk_size[100];
+  size_to_unit(disk_car->disk_size, buffer_disk_size);
   snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - %s - CHS %lu %u %u%s",
-      disk_car->device, size_to_unit(disk_car->disk_size,buffer_disk_size),
+      disk_car->device, buffer_disk_size,
       disk_car->geom.cylinders, disk_car->geom.heads_per_cylinder, disk_car->geom.sectors_per_head,
       ((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
   return disk_car->description_txt;
@@ -1034,13 +1080,14 @@ static const char *file_description_short(disk_t *disk_car)
 {
   const struct info_file_struct *data=(const struct info_file_struct *)disk_car->data;
   char buffer_disk_size[100];
+  size_to_unit(disk_car->disk_size, buffer_disk_size);
   if(disk_car->model==NULL)
     snprintf(disk_car->description_short_txt, sizeof(disk_car->description_txt),"Disk %s - %s%s",
-      disk_car->device, size_to_unit(disk_car->disk_size,buffer_disk_size),
+      disk_car->device, buffer_disk_size,
       ((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
   else
     snprintf(disk_car->description_short_txt, sizeof(disk_car->description_txt),"Disk %s - %s%s - %s",
-      disk_car->device, size_to_unit(disk_car->disk_size,buffer_disk_size),
+      disk_car->device, buffer_disk_size,
       ((data->mode&O_RDWR)==O_RDWR?"":" (RO)"),
       disk_car->model);
   return disk_car->description_short_txt;
@@ -1060,6 +1107,13 @@ static int file_clean(disk_t *disk_car)
     }
 #endif
     */
+#ifdef HDCLONE
+    if(data->handle_clone>0)
+    {
+      close(data->handle_clone);
+      data->handle_clone=0;
+    }
+#endif
     close(data->handle);
   }
   return generic_clean(disk_car);
@@ -1130,6 +1184,17 @@ static int file_pread_aux(disk_t *disk_car, void *buf, const unsigned int count,
     }
     memset((char*)buf+ret,0,count-ret);
   }
+#ifdef HDCLONE
+  else
+  {
+    int handle_clone=((struct info_file_struct *)disk_car->data)->handle_clone;
+    if(handle_clone>0)
+    {
+      pwrite(handle_clone, buf, count, offset);
+      fdatasync(handle_clone);
+    }
+  }
+#endif
   return ret;
 }
 
@@ -1138,6 +1203,13 @@ static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, con
   return align_pread(&file_pread_aux, disk_car, buf, count, offset);
 }
 
+static void *file_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
+{
+  if(file_pread(disk, buf, count, offset)==offset)
+    return buf;
+  return NULL;
+}
+
 static int file_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   int fd=((struct info_file_struct *)disk_car->data)->handle;
@@ -1192,8 +1264,8 @@ static int file_nopwrite(disk_t *disk_car, const void *buf, const unsigned int c
 
 static int file_sync(disk_t *disk_car)
 {
-  struct info_file_struct *data=(struct info_file_struct *)disk_car->data;
 #ifdef HAVE_FSYNC
+  struct info_file_struct *data=(struct info_file_struct *)disk_car->data;
   return fsync(data->handle);
 #else
   errno=EINVAL;
@@ -1201,31 +1273,37 @@ static int file_sync(disk_t *disk_car)
 #endif
 }
 
-static void autoset_geometry(disk_t * disk_car, const unsigned char *buffer, const int verbose)
+static void autoset_geometry(disk_t *disk, const unsigned char *buffer, const int verbose)
 {
-  if(disk_car->arch->get_geometry_from_mbr!=NULL)
+  if(disk->arch->get_geometry_from_mbr!=NULL)
   {
     CHSgeometry_t geometry;
     geometry.cylinders=0;
     geometry.heads_per_cylinder=0;
     geometry.sectors_per_head=0;
-    disk_car->arch->get_geometry_from_mbr(buffer, verbose, &geometry);
-    disk_car->autodetect=1;
+    geometry.bytes_per_sector=0;
+    disk->arch->get_geometry_from_mbr(buffer, verbose, &geometry);
+    disk->autodetect=1;
     if(geometry.sectors_per_head > 0)
     {
-      disk_car->geom.heads_per_cylinder=geometry.heads_per_cylinder;
-      disk_car->geom.sectors_per_head=geometry.sectors_per_head;
+      disk->geom.heads_per_cylinder=geometry.heads_per_cylinder;
+      disk->geom.sectors_per_head=geometry.sectors_per_head;
+      if(geometry.bytes_per_sector!=0)
+      {
+	disk->geom.bytes_per_sector=geometry.bytes_per_sector;
+	disk->sector_size=geometry.bytes_per_sector;
+      }
     }
     else
     {
-      disk_car->geom.heads_per_cylinder=255;
-      disk_car->geom.sectors_per_head=63;
+      disk->geom.heads_per_cylinder=255;
+      disk->geom.sectors_per_head=63;
     }
   }
   /* Round up because file is often truncated. */
-  disk_car->geom.cylinders=(disk_car->disk_size / disk_car->sector_size +
-      (uint64_t)disk_car->geom.sectors_per_head * disk_car->geom.heads_per_cylinder - 1) /
-    disk_car->geom.sectors_per_head / disk_car->geom.heads_per_cylinder;
+  disk->geom.cylinders=(disk->disk_size / disk->sector_size +
+      (uint64_t)disk->geom.sectors_per_head * disk->geom.heads_per_cylinder - 1) /
+    disk->geom.sectors_per_head / disk->geom.heads_per_cylinder;
 }
 
 disk_t *file_test_availability(const char *device, const int verbose, const arch_fnct_t *arch, int testdisk_mode)
@@ -1357,6 +1435,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   disk_car->data=data;
   disk_car->description=file_description;
   disk_car->description_short=file_description_short;
+  disk_car->pread_fast=file_pread_fast;
   disk_car->pread=file_pread;
   disk_car->pwrite=((mode&O_RDWR)==O_RDWR?file_pwrite:file_nopwrite);
   disk_car->sync=file_sync;
@@ -1397,11 +1476,13 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
 #endif
   {
     unsigned char *buffer;
+    struct tdewf_file_header *ewf;
     const uint8_t evf_file_signature[8] = { 'E', 'V', 'F', 0x09, 0x0D, 0x0A, 0xFF, 0x00 };
     if(verbose>1)
       log_verbose("file_test_availability %s is a file\n", device);
     disk_car->sector_size=DEFAULT_SECTOR_SIZE;
     buffer=(unsigned char*)MALLOC(DEFAULT_SECTOR_SIZE);
+    ewf=(struct tdewf_file_header*)buffer;
     if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)<0)
     {
       memset(buffer,0,DEFAULT_SECTOR_SIZE);
@@ -1453,10 +1534,25 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   }
   update_disk_car_fields(disk_car);
 #if defined(POSIX_FADV_SEQUENTIAL) && defined(HAVE_POSIX_FADVISE)
-  posix_fadvise(hd_h,0,0,POSIX_FADV_SEQUENTIAL);
+//  posix_fadvise(hd_h,0,0,POSIX_FADV_SEQUENTIAL);
 #endif
   if(disk_car->disk_real_size!=0)
+  {
+#ifdef HDCLONE
+    if(strncmp(device, "/dev/", 5)==0)
+    {
+      char *new_file=(char *)MALLOC(strlen(device)+5);
+      sprintf(new_file, "%s.dd", device);
+#ifdef O_LARGEFILE
+      data->handle_clone=open(new_file, O_CREAT|O_EXCL|O_LARGEFILE|O_WRONLY,00600);
+#else
+      data->handle_clone=open(new_file, O_CREAT|O_EXCL|O_WRONLY,00600);
+#endif
+      free(new_file);
+    }
+#endif
     return disk_car;
+  }
   if(disk_car->model==NULL)
     log_warning("Warning: can't get size for %s, sector size=%u\n",
 	disk_car->description(disk_car), disk_car->sector_size);
@@ -1471,26 +1567,26 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   return NULL;
 }
 
-void hd_update_geometry(disk_t *disk_car, const int allow_partial_last_cylinder, const int verbose)
+void hd_update_geometry(disk_t *disk, const int allow_partial_last_cylinder, const int verbose)
 {
-  unsigned char *buffer;
-  buffer=(unsigned char *)MALLOC(disk_car->sector_size);
-  if(disk_car->autodetect!=0)
+  if(disk->autodetect!=0)
   {
-    if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, 0) == disk_car->sector_size)
+    unsigned char *buffer=(unsigned char *)MALLOC(disk->sector_size);
+    if((unsigned)disk->pread(disk, buffer, disk->sector_size, 0) == disk->sector_size)
     {
       if(verbose>1)
       {
 	log_trace("autoset_geometry\n");
       }
-      autoset_geometry(disk_car,buffer,1);
+      autoset_geometry(disk,buffer,1);
     }
+    free(buffer);
   }
-  if((unsigned int)(disk_car->geom.cylinders+1)!=0)	/* Avoid to wrap */
+  if((unsigned int)(disk->geom.cylinders+1)!=0)	/* Avoid to wrap */
   {
     uint64_t pos;
     CHS_t pos_CHS;
-    pos_CHS.cylinder=disk_car->geom.cylinders-1+1;
+    pos_CHS.cylinder=disk->geom.cylinders-1+1;
     if(allow_partial_last_cylinder)
     {
       pos_CHS.head=0;
@@ -1498,35 +1594,38 @@ void hd_update_geometry(disk_t *disk_car, const int allow_partial_last_cylinder,
     }
     else
     {
-      pos_CHS.head=disk_car->geom.heads_per_cylinder-1;
-      pos_CHS.sector=disk_car->geom.sectors_per_head;
+      pos_CHS.head=disk->geom.heads_per_cylinder-1;
+      pos_CHS.sector=disk->geom.sectors_per_head;
     }
-    pos=CHS2offset(disk_car,&pos_CHS);
+    pos=CHS2offset(disk,&pos_CHS);
 #ifdef DJGPP
-    if(disk_car->description==disk_description)
+    if(disk->description==disk_description)
     {
-      struct info_disk_struct*data=disk_car->data;
+      struct info_disk_struct*data=disk->data;
       data->geo_phys.cylinders=0;
     }
 #endif
-    if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, pos) == disk_car->sector_size)
     {
-      disk_car->geom.cylinders++;
-      if(disk_car->disk_size < (uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size)
+      unsigned char *buffer=(unsigned char *)MALLOC(disk->sector_size);
+      if((unsigned)disk->pread(disk, buffer, disk->sector_size, pos) == disk->sector_size)
       {
-	disk_car->disk_size = (uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
-	log_info("Computes LBA from CHS for %s\n",disk_car->description(disk_car));
+	disk->geom.cylinders++;
+	if(disk->disk_size < (uint64_t)disk->geom.cylinders * disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size)
+	{
+	  disk->disk_size = (uint64_t)disk->geom.cylinders * disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size;
+	  log_info("Computes LBA from CHS for %s\n",disk->description(disk));
+	}
       }
+      free(buffer);
     }
   }
 #ifdef DJGPP
-  if(disk_car->description==disk_description)
+  if(disk->description==disk_description)
   {
-    struct info_disk_struct*data=disk_car->data;
-    data->geo_phys.cylinders=disk_car->geom.cylinders;
+    struct info_disk_struct*data=disk->data;
+    data->geo_phys.cylinders=disk->geom.cylinders;
   }
 #endif
-  free(buffer);
 }
 
 void hd_update_all_geometry(const list_disk_t * list_disk, const int allow_partial_last_cylinder, const int verbose)
@@ -1540,22 +1639,13 @@ void hd_update_all_geometry(const list_disk_t * list_disk, const int allow_parti
     hd_update_geometry(element_disk->disk,allow_partial_last_cylinder,verbose);
 }
 
-void autoset_unit(disk_t *disk_car)
-{
-  if(disk_car==NULL)
-    return ;
-  if(disk_car->arch==&arch_mac || 
-      disk_car->arch==&arch_gpt || 
-      (disk_car->geom.heads_per_cylinder==1 && disk_car->geom.sectors_per_head==1))
-    disk_car->unit=UNIT_SECTOR;
-  else
-    disk_car->unit=UNIT_CHS;
-}
-
 void init_disk(disk_t *disk)
 {
   disk->autodetect=0;
   disk->disk_size=0;
+  disk->user_max=0;
+  disk->native_max=0;
+  disk->dco=0;
   /* Note, some Raid reserve the first 1024 512-sectors */
   disk->offset=0;
   disk->rbuffer=NULL;
