@@ -63,6 +63,10 @@
 #include <ntfs/version.h>
 #endif
 #endif
+#ifdef HAVE_LIBNTFS3G
+#include <ntfs-3g/volume.h>
+#include <ntfs-3g/attrib.h>
+#endif
 
 #include "common.h"
 #include "intrf.h"
@@ -72,8 +76,9 @@
 #include "ntfs_utl.h"
 #include "ntfs_inc.h"
 #include "log.h"
+#include "setdate.h"
 
-#ifdef HAVE_LIBNTFS
+#if defined(HAVE_LIBNTFS) || defined(HAVE_LIBNTFS3G)
 #define MAX_PATH    1024
 #define PATH_SEP      '/'
 #define NTFS_DT_DIR               4
@@ -97,9 +102,7 @@ extern struct ntfs_device_operations ntfs_device_testdisk_io_ops;
 
 extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
                 void *dirent, ntfs_filldir_t filldir);
-extern u64 ntfs_inode_lookup_by_name(ntfs_inode *dir_ni,
-                const ntfschar *uname, const int uname_len);
-static time_t ntfs2utc (s64 ntfstime);
+static time_t td_ntfs2utc (s64 ntfstime);
 static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
@@ -131,7 +134,7 @@ static int index_get_size(ntfs_inode *inode)
 }
 
 /**
- * ntfs2utc - Convert an NTFS time to Unix time
+ * td_ntfs2utc - Convert an NTFS time to Unix time
  * @time:  An NTFS time in 100ns units since 1601
  *
  * NTFS stores times as the number of 100ns intervals since January 1st 1601 at
@@ -139,7 +142,7 @@ static int index_get_size(ntfs_inode *inode)
  *
  * Return:  n  A Unix time (number of seconds since 1970)
  */
-static time_t ntfs2utc (s64 ntfstime)
+static time_t td_ntfs2utc (s64 ntfstime)
 {
   return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
 }
@@ -179,7 +182,7 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
 {
   char *filename = NULL;
   int result = 0;
-  filename = calloc (1, MAX_PATH);
+  filename = (char *)calloc (1, MAX_PATH);
   if (!filename)
   {
     log_critical("ntfs_td_list_entry calloc failed\n");
@@ -209,8 +212,8 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
     s64 filesize = 0;
     ntfs_inode *ni;
     ntfs_attr_search_ctx *ctx = NULL;
-    FILE_NAME_ATTR *file_name_attr;
     ATTR_RECORD *attr;
+    STANDARD_INFORMATION *si;
 
     result = -1;				/* Everything else is bad */
 
@@ -221,17 +224,16 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
     ctx = ntfs_attr_get_search_ctx(ni, ni->mrec);
     if (!ctx)
       goto release;
-
-    if (ntfs_attr_lookup(AT_FILE_NAME, AT_UNNAMED, 0, 0, 0, NULL,
+    if (ntfs_attr_lookup(AT_STANDARD_INFORMATION, AT_UNNAMED, 0, 0, 0, NULL,
 	  0, ctx))
       goto release;
     attr = ctx->attr;
 
-    file_name_attr = (FILE_NAME_ATTR *)((char *)attr +
-	le16_to_cpu(attr->value_offset));
-    if (!file_name_attr)
-      goto release;
+    si = (STANDARD_INFORMATION*)((char*)attr +
+                            le16_to_cpu(attr->value_offset));
 
+    if (!si)
+      goto release;
 
     if (dt_type != NTFS_DT_DIR) {
       if (!ntfs_attr_lookup(AT_DATA, AT_UNNAMED, 0, 0, 0,
@@ -246,24 +248,27 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
       new_file->status=0;
       new_file->prev=ls->current_file;
       new_file->next=NULL;
-      new_file->filestat.st_dev=0;
-      new_file->filestat.st_ino=MREF(mref);
-      new_file->filestat.st_mode = (dt_type == NTFS_DT_DIR?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
-      new_file->filestat.st_nlink=1;
-      new_file->filestat.st_uid=0;
-      new_file->filestat.st_gid=0;
-      new_file->filestat.st_rdev=0;
-      new_file->filestat.st_size=filesize;
-      new_file->filestat.st_blksize=DEFAULT_SECTOR_SIZE;
+      new_file->stat.st_dev=0;
+      new_file->stat.st_ino=MREF(mref);
+      new_file->stat.st_mode = (dt_type == NTFS_DT_DIR?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
+      new_file->stat.st_nlink=1;
+      new_file->stat.st_uid=0;
+      new_file->stat.st_gid=0;
+      new_file->stat.st_rdev=0;
+      new_file->stat.st_size=filesize;
+#ifdef DJGPP
+      new_file->file_size=filesize;
+#endif
+      new_file->stat.st_blksize=DEFAULT_SECTOR_SIZE;
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-      if(new_file->filestat.st_blksize!=0)
+      if(new_file->stat.st_blksize!=0)
       {
-	new_file->filestat.st_blocks=(new_file->filestat.st_size+new_file->filestat.st_blksize-1)/new_file->filestat.st_blksize;
+	new_file->stat.st_blocks=(filesize + new_file->stat.st_blksize - 1) / new_file->stat.st_blksize;
       }
 #endif
-      new_file->filestat.st_atime=ntfs2utc(sle64_to_cpu(file_name_attr->last_access_time));
-      new_file->filestat.st_ctime=ntfs2utc(sle64_to_cpu(file_name_attr->creation_time));
-      new_file->filestat.st_mtime=ntfs2utc(sle64_to_cpu(file_name_attr->last_data_change_time));
+      new_file->stat.st_atime=td_ntfs2utc(sle64_to_cpu(si->last_access_time));
+      new_file->stat.st_mtime=td_ntfs2utc(sle64_to_cpu(si->last_data_change_time));
+      new_file->stat.st_ctime=td_ntfs2utc(sle64_to_cpu(si->creation_time));
       new_file->prev=ls->current_file;
       new_file->next=NULL;
       /* log_debug("fat: new file %s de=%p size=%u\n",new_file->name,de,de->size); */
@@ -319,9 +324,11 @@ static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir
   return ls->dir_list;
 }
 
+enum { bufsize = 4096 };
+
 static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
 {
-  const unsigned long int first_inode=file->filestat.st_ino;
+  const unsigned long int first_inode=file->stat.st_ino;
   ntfs_inode *inode;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
   inode = ntfs_inode_open (ls->vol, first_inode);
@@ -330,7 +337,6 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     return -1;
   }
   {
-    const int bufsize = 4096;
     char *buffer;
     char *new_file;
     ntfs_attr *attr;
@@ -340,12 +346,16 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     u32 block_size;
     buffer = (char *)MALLOC(bufsize);
     if (!buffer)
+    {
+      ntfs_inode_close(inode);
       return -2;
+    }
     attr = ntfs_attr_open(inode, AT_DATA, NULL, 0);
     if (!attr)
     {
       log_error("Cannot find attribute type 0x%lx.\n", (long) AT_DATA);
       free(buffer);
+      ntfs_inode_close(inode);
       return -3;
     }
     if ((inode->mft_no < 2) && (attr->type == AT_DATA))
@@ -354,12 +364,14 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       block_size = index_get_size(inode);
     else
       block_size = 0;
-    new_file=gen_local_filename(dir_data->local_dir, dir_data->current_directory);
-    f_out=create_file(new_file);
+    f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
     if(!f_out)
     {
       log_critical("Can't create file %s: %s\n",new_file, strerror(errno));
       free(new_file);
+      ntfs_attr_close(attr);
+      free(buffer);
+      ntfs_inode_close(inode);
       return -4;
     }
     offset = 0;
@@ -389,9 +401,9 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       offset += bytes_read;
     }
     fclose(f_out);
-    set_date(new_file, file->filestat.st_atime, file->filestat.st_mtime);
-    ntfs_attr_close(attr);
+    set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
     free(new_file);
+    ntfs_attr_close(attr);
     free(buffer);
   }
   /* Finished with the inode; release it. */
@@ -401,12 +413,8 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
 
 static void dir_partition_ntfs_close(dir_data_t *dir_data)
 {
-  const partition_t *partition;
-  disk_t *disk_car;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
   /* ntfs_umount() will invoke ntfs_device_free() for us. */
-  disk_car=ls->my_data->disk_car;
-  partition=ls->my_data->partition;
   ntfs_umount(ls->vol, FALSE);
   free(ls->my_data);
 #ifdef HAVE_ICONV
@@ -419,7 +427,7 @@ static void dir_partition_ntfs_close(dir_data_t *dir_data)
 
 int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
-#ifdef HAVE_LIBNTFS
+#if defined(HAVE_LIBNTFS) || defined(HAVE_LIBNTFS3G)
   struct ntfs_device *dev;
   my_data_t *my_data=NULL;
   ntfs_volume *vol=NULL;
@@ -501,6 +509,8 @@ const char*td_ntfs_version(void)
 #else
   return "available";
 #endif
+#elif defined(HAVE_LIBNTFS3G)
+  return "libntfs-3g";
 #else
   return "none";
 #endif
