@@ -23,6 +23,8 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -36,14 +38,16 @@
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
-#include <stdio.h>
 #include "dir.h"
 #include "filegen.h"
 #include "photorec.h"
+#include "exfatp.h"
 #include "ext2p.h"
 #include "fatp.h"
 #include "ntfsp.h"
 #include "log.h"
+#include "setdate.h"
+#include "dfxml.h"
 
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
@@ -52,6 +56,7 @@
 static void update_search_space(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize);
 static void update_search_space_aux(alloc_data_t *list_search_space, uint64_t start, uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset);
 static alloc_data_t *file_truncate(alloc_data_t *space, file_recovery_t *file, const unsigned int sector_size, const unsigned int blocksize);
+static alloc_data_t *file_error(alloc_data_t *space, file_recovery_t *file, const unsigned int blocksize);
 static void list_free_add(const file_recovery_t *file_recovery, alloc_data_t *list_search_space);
 static void list_space_used(const file_recovery_t *file_recovery, const unsigned int sector_size);
 
@@ -294,6 +299,12 @@ void free_list_search_space(alloc_data_t *list_search_space)
   }
 }
 
+/** 
+ * @param recup_dir - base name of output directory
+ * @param initial_dir_num - first number to try appending.
+ * @return the number that was appended.
+ */
+
 unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_dir_num)
 {
   char working_recup_dir[2048];
@@ -338,11 +349,15 @@ int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current
 {
   int nbr;
   alloc_data_t *file_space=*current_search_space;
-  for(nbr=0;nbr<10;nbr++)
+  uint64_t size=0;
+  /* Search backward the first fragment of a file not successfully recovered
+   * Limit the search to 10 fragments or 1GB */
+  for(nbr=0; nbr<10 && size < (uint64_t)1024*1024*1024; nbr++)
   {
     file_space=td_list_entry(file_space->list.prev, alloc_data_t, list);
     if(file_space==list_search_space)
       return -1;
+    size+=file_space->end - file_space->start + 1;
     if(file_space->file_stat!=NULL)
     {
       *current_search_space=file_space;
@@ -379,9 +394,13 @@ void forget(alloc_data_t *list_search_space, alloc_data_t *current_search_space)
 
 unsigned int remove_used_space(disk_t *disk_car, const partition_t *partition, alloc_data_t *list_search_space)
 {
-  if(partition->upart_type==UP_FAT16 || partition->upart_type==UP_FAT32)
+  if( partition->upart_type==UP_FAT12 ||
+      partition->upart_type==UP_FAT16 || 
+      partition->upart_type==UP_FAT32)
     return fat_remove_used_space(disk_car, partition, list_search_space);
-#ifdef HAVE_LIBNTFS
+  else if(partition->upart_type==UP_EXFAT)
+    return exfat_remove_used_space(disk_car, partition, list_search_space);
+#if defined(HAVE_LIBNTFS) || defined(HAVE_LIBNTFS3G)
   else if(partition->upart_type==UP_NTFS)
     return ntfs_remove_used_space(disk_car, partition, list_search_space);
 #endif
@@ -493,16 +512,6 @@ partition_t *new_whole_disk(const disk_t *disk_car)
   return fake_partition;
 }
 
-
-typedef struct info_cluster_offset cluster_offset_t;
-
-struct info_cluster_offset
-{
-  unsigned int cluster_size;
-  unsigned long int offset;
-  unsigned int nbr;
-};
-
 unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int default_blocksize, uint64_t *offset)
 {
   unsigned int blocksize=128*512;
@@ -550,6 +559,8 @@ void update_blocksize(unsigned int blocksize, alloc_data_t *list_search_space, c
   }
 }
 
+uint64_t free_list_allocation_end=0;
+
 static void free_list_allocation(alloc_list_t *list_allocation)
 {
   struct td_list_head *tmp = NULL;
@@ -558,12 +569,26 @@ static void free_list_allocation(alloc_list_t *list_allocation)
   {
     alloc_list_t *allocated_space;
     allocated_space=td_list_entry(tmp, alloc_list_t, list);
+    free_list_allocation_end=allocated_space->end;
     td_list_del(tmp);
     free(allocated_space);
   }
 }
 
-/* file_finish() returns
+/** file_finish()
+    @param file_recovery - 
+    @param recup_dir - where the results go
+    @param paranoid
+    @param file_nbr
+    @param blocksize
+    @param alloc_data_t *list_search_space
+    @param alloc_data_t **current_search_space
+    @param *offset
+    @param dir_num
+    @param status
+    @param dist_t *disk
+
+    @returns:
    -1: file not recovered, file_size=0 offset_error!=0
     0: file not recovered
     1: file recovered
@@ -614,7 +639,12 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
     //    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
-    if(file_recovery->file_size>0)
+    if(file_recovery->file_size==0)
+    {
+      /* File is zero-length; erase it */
+      unlink(file_recovery->filename);
+    }
+    else
     {
       if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
 	set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
@@ -625,16 +655,10 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
       if(status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
         file_recovery->file_stat->recovered++;
     }
-    else
-    {
-      unlink(file_recovery->filename);
-    }
   }
   if(file_recovery->file_stat!=NULL)
   {
     list_truncate(&file_recovery->location,file_recovery->file_size);
-    if(file_recovery->file_size>0)
-      list_space_used(file_recovery, disk->sector_size);
     if(file_recovery->file_size==0)
     {
       /* File hasn't been sucessfully recovered, remember where it begins */
@@ -643,10 +667,15 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
           !((*current_search_space)->start <= *offset && *offset <= (*current_search_space)->end))
         *current_search_space=td_list_entry((*current_search_space)->list.next, alloc_data_t, list);
     }
-    else if(status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING && status!=STATUS_FIND_OFFSET)
+    else
     {
-      update_search_space(file_recovery,list_search_space,current_search_space,offset,blocksize);
-      file_recovered=1;
+      list_space_used(file_recovery, disk->sector_size);
+      xml_log_file_recovered(file_recovery);
+      if(status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING && status!=STATUS_FIND_OFFSET)
+      {
+	update_search_space(file_recovery,list_search_space,current_search_space,offset,blocksize);
+	file_recovered=1;			/* note that file was recovered */
+      }
     }
     free_list_allocation(&file_recovery->location);
   }
@@ -706,8 +735,11 @@ alloc_data_t *file_finish2(file_recovery_t *file_recovery, const char *recup_dir
     }
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
-    //    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
-    if(file_recovery->file_size>0)
+    if(file_recovery->file_size==0)
+    {
+      unlink(file_recovery->filename);
+    }
+    else
     {
       if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
 	set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
@@ -718,11 +750,9 @@ alloc_data_t *file_finish2(file_recovery_t *file_recovery, const char *recup_dir
         *dir_num=photorec_mkdir(recup_dir,*dir_num+1);
       }
       if(status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
+      {
         file_recovery->file_stat->recovered++;
-    }
-    else
-    {
-      unlink(file_recovery->filename);
+      }
     }
   }
   if(file_recovery->file_stat!=NULL)
@@ -730,9 +760,12 @@ alloc_data_t *file_finish2(file_recovery_t *file_recovery, const char *recup_dir
     if(file_recovery->file_size==0)
     {
       /* File hasn't been sucessfully recovered */
+      if(file_recovery->offset_error>0)
+	datanext=file_error(list_search_space, file_recovery, blocksize);
     }
     else
     {
+      xml_log_file_recovered2(list_search_space, file_recovery);
       datanext=file_truncate(list_search_space, file_recovery, disk->sector_size, blocksize);
     }
     free_list_allocation(&file_recovery->location);
@@ -793,9 +826,10 @@ static alloc_data_t *file_truncate_aux(alloc_data_t *space, alloc_data_t *file,
       return element;
     if(element->data>0)
     {
-      if(size + (element->end-element->start+1) <= file_size_on_disk)
+      const uint64_t len=element->end - element->start + 1;
+      if(size + len <= file_size_on_disk)
       {
-	size=size + (element->end-element->start+1);
+	size+=len;
 	log_info(" %lu-%lu", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
 	td_list_del(tmp);
 	free(element);
@@ -835,6 +869,59 @@ static alloc_data_t *file_truncate(alloc_data_t *space, file_recovery_t *file, c
   return datanext;
 }
 
+static alloc_data_t *file_error_aux(alloc_data_t *space, alloc_data_t *file, const uint64_t file_size, const unsigned int blocksize)
+{
+  struct td_list_head *tmp;
+  struct td_list_head *next;
+  uint64_t size=0;
+  const uint64_t file_size_on_disk=file_size/blocksize*blocksize;
+  for(tmp=&file->list, next=tmp->next; tmp!=&space->list; tmp=next, next=tmp->next)
+  {
+    alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
+    if(size >= file_size)
+      return NULL;
+    if(element->data>0)
+    {
+      if(size + (element->end-element->start+1) <= file_size_on_disk)
+      {
+	size=size + (element->end-element->start+1);
+      }
+      else if(file_size_on_disk > size)
+      {
+	if(element->file_stat==NULL)
+	  return NULL;
+	if(next!=&space->list)
+	{
+	  alloc_data_t *new_element;
+	  new_element=td_list_entry(next, alloc_data_t, list);
+	  if(element->end+1==new_element->start && new_element->file_stat==NULL)
+	  {
+	    new_element->start-=file_size_on_disk - size;
+	    element->end=new_element->start - 1;
+	    return new_element;
+	  }
+	}
+	{
+	  alloc_data_t *new_element;
+	  new_element=(alloc_data_t*)MALLOC(sizeof(*new_element));
+	  memcpy(new_element, element, sizeof(*new_element));
+	  new_element->start+=file_size_on_disk - size;
+	  new_element->file_stat=NULL;
+	  td_list_add(&new_element->list, &element->list);
+	  element->end=new_element->start - 1;
+	  return new_element;
+	}
+      }
+    }
+  }
+  return NULL;
+}
+
+static alloc_data_t *file_error(alloc_data_t *space, file_recovery_t *file, const unsigned int blocksize)
+{
+  return file_error_aux(space, file->loc, file->offset_error, blocksize);
+}
+
 void free_search_space(alloc_data_t *list_search_space)
 {
   struct td_list_head *search_walker = NULL;
@@ -847,3 +934,20 @@ void free_search_space(alloc_data_t *list_search_space)
     free(current_search_space);
   }
 }
+
+void set_filename(file_recovery_t *file_recovery, const char *recup_dir, const unsigned int dir_num, const disk_t *disk, const partition_t *partition, const int broken)
+{
+  if(file_recovery->extension==NULL || file_recovery->extension[0]=='\0')
+  {
+    snprintf(file_recovery->filename,sizeof(file_recovery->filename)-1,"%s.%u/%c%07u",recup_dir,
+	dir_num,(broken?'b':'f'),
+	(unsigned int)((file_recovery->location.start-partition->part_offset)/disk->sector_size));
+  }
+  else
+  {
+    snprintf(file_recovery->filename,sizeof(file_recovery->filename)-1,"%s.%u/%c%07u.%s",recup_dir,
+	dir_num, (broken?'b':'f'),
+	(unsigned int)((file_recovery->location.start-partition->part_offset)/disk->sector_size), file_recovery->extension);
+  }
+}
+
